@@ -2,7 +2,7 @@
  *
  * Reads a (private) key on stdin, and saves it a private and a public file.
  *
- * $Id: lsh-writekey.c,v 1.3 2000/09/14 23:46:18 nisse Exp $ */
+ * $Id: lsh-writekey.c,v 1.4 2000/10/17 23:45:18 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -383,7 +383,10 @@ COMMAND_SIMPLE(lsh_writekey_options2private_file)
        (lambda (backend)
          (let ((key (read_sexp (stdin backend))))
            (prog1 (print_public options (open backend (options2public_file options))
-	                        (signer2public (sexp2signer (options2algorithms options) key)))
+	                        (verifier2public
+				  (signer2verifier
+				    (sexp2signer (options2algorithms options)
+				                 key))))
 	          (print_private options (open backend (options2private_file options))
 		                 (transform options key)))))))
 */
