@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.118 2000/09/14 23:48:23 nisse Exp $ */
+ * $Id: lsh.c,v 1.119 2000/10/17 21:52:18 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -387,15 +387,15 @@ do_lsh_lookup(struct lookup_verifier *c,
     {
     case ATOM_SSH_DSS:
       {
-	struct dsa_verifier *v = make_ssh_dss_verifier(key->length, key->data);
+	struct verifier *v = make_ssh_dss_verifier(key->length, key->data);
 	if (!v)
 	  {
 	    werror("do_lsh_lookup: Invalid ssh-dss key.\n");
 	    return NULL;
 	  }
 	subject = SPKI_LOOKUP(self->db,
-			      dsa_to_spki_public_key(&v->public),
-			      &v->super);
+			      spki_make_public_key(v),
+			      v);
 	assert(subject);
 	assert(subject->verifier);
 	break;
