@@ -2,7 +2,7 @@
  *
  * main proxy program.
  *
- * $Id: lsh_proxy.c,v 1.13 2000/09/14 23:48:24 nisse Exp $ */
+ * $Id: lsh_proxy.c,v 1.14 2000/10/17 21:54:02 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -412,8 +412,8 @@ do_host_lookup(struct lookup_verifier *c UNUSED,
 {
   switch(method)
     {
-      case ATOM_SSH_DSS:
-	return &make_ssh_dss_verifier(key->length, key->data)->super;
+    case ATOM_SSH_DSS:
+	return make_ssh_dss_verifier(key->length, key->data);
     default:
       return NULL;
     }
