@@ -1,6 +1,6 @@
 /* tcpforward.h
  *
- * $Id: tcpforward.h,v 1.14 2000/04/21 22:29:46 nisse Exp $
+ * $Id: tcpforward.h,v 1.15 2000/08/04 00:17:21 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -42,7 +42,7 @@
        ; this could store the type of this forward
        ; tcp, udp etc. Or we could invent relevant methods
        ; and subclasses.
-       ; (type simple UINT32)
+       ; (type . int)
        
        (listen object address_info)))
 */
