@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: write_buffer.h,v 1.18 2000/08/04 00:17:22 nisse Exp $ */
+ * $Id: write_buffer.h,v 1.19 2000/10/06 23:26:30 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -44,14 +44,6 @@
 
 #define FLOW_CONTROL_REPORT(o, written) ((o)->report((o), written))
 
-/* For the packet queue */
-/* NOTE: No object header */
-struct buffer_node
-{
-  struct lsh_queue_node header;
-  struct lsh_string *packet;
-};
-
 /* GABA:
    (class
      (name write_buffer)
@@ -72,8 +64,7 @@ struct buffer_node
 
        ;; (try_write . int)
 
-       (q indirect-special "struct lsh_queue"
-                           #f do_free_buffer)
+       (q struct string_queue)
 
        (pos . UINT32)        ; Partial packet
        (partial string)
